(* Solution to transport problem IOI '94 *)
(* Written by Stefan Burstrm and Erik Persson *)

PROGRAM Transp;

CONST CITYCNT = 7;
      ENDCITY = 1;
      MAXORDERS = 32;

TYPE DistType = Array[1..CITYCNT,1..CITYCNT] OF Integer;
     OrderType = Array[1..MAXORDERS,1..2] OF Integer;
     CitiesType = Array[1..CITYCNT] OF INTEGER;

CONST
  Distances :DistType=           (* This is 'DISTANCE.TXT' *)
  (  (00, 56, 43, 71, 35, 41, 36),
     (56, 00, 54, 58, 36, 79, 31),
     (43, 54, 00, 30, 20, 31, 58),
     (71, 58, 30, 00, 38, 59, 75),
     (35, 36, 20, 38, 00, 44, 40),
     (41, 79, 31, 59, 44, 00, 72),
     (36, 31, 58, 75, 40, 72, 00)
  );

VAR i,j,OrderCnt,OrdersLeft :INTEGER;
    Orders :OrderType;
    ToFrom,FromCnt :CitiesType;
    MinWay,
    XWay,BestXWay :INTEGER;
    Answer,BestAnswer :String;
    f :TEXT;

PROCEDURE ReadOrders(name :STRING;
                    VAR n :INTEGER;
                    VAR Orders :OrderType);

VAR f :TEXT;
    i,j :INTEGER;
    c :char;
BEGIN
 Assign(f,name);
 Reset(f);
 ReadLn(f,n);
 FOR i:=1 TO n DO
  BEGIN
   Read(f,c);
   Orders[i,1]:= ord(c)-64;
   Read(f,c);
   ReadLn(f,c);
   Orders[i,2]:= ord(c)-64;
  END;
 Close(f);
END;

PROCEDURE CalcTables(VAR Orders :OrderType;
                     VAR FromCnt,ToFrom :CitiesType;
                     VAR MinWay :INTEGER;
                     OrderCnt :INTEGER);

VAR i,j :INTEGER;

BEGIN (* Calculate MinWay: minimal way
                   FromCnt[1..CITYCNT]: # of orders from city
                   ToFrom[1..CITYCNT]: # of orders to city - # of orders from city *)

 FillChar(ToFrom,SizeOf(ToFrom),0);
 FillChar(FromCnt,SizeOf(FromCnt),0);
 MinWay:=0;
 FOR i:=1 TO OrderCnt DO
  BEGIN
   Inc(ToFrom[Orders[i,1]]);
   Dec(ToFrom[Orders[i,2]]);
   Inc(FromCnt[Orders[i,1]]);

   MinWay := MinWay+Distances[Orders[i,1],Orders[i,2]];
  END;
END;

PROCEDURE Solve(city,
                fromcity,
                level :INTEGER;  (* level of recursion *)
                cargo :Boolean); (* Did we deliver cargo to 'city'? *)


(* On entry of this procedure a flight has been made from 'fromcity'
   to 'city'. *)

VAR i,saveXWay :INTEGER;
    done :Array[1..CITYCNT] OF Boolean;

BEGIN
 saveXWay:=XWay;
   Answer[level*2+2]:=Chr(city+64);
   IF NOT cargo THEN
    BEGIN
     XWay:=XWay+Distances[city,fromcity];
     IF XWay>=BestXWay THEN
      BEGIN
       XWay:=saveXWay;
       EXIT;
      END;
    END;
   IF OrdersLeft>0 THEN
    BEGIN
     FillChar(done,SizeOf(done),0);
     FOR i:=1 TO OrderCnt DO
      BEGIN
       IF Orders[i,1]=city THEN
        BEGIN
         Orders[i,1]:=-1;
         Dec(FromCnt[city]);
         done[Orders[i,2]]:=TRUE;
         Dec(OrdersLeft);
         Solve(Orders[i,2],city,level+1,true);
         Inc(OrdersLeft);
         Inc(FromCnt[city]);
         Orders[i,1]:=city;
        END;
      END;
     IF cargo AND (ToFrom[city]<=0) THEN
      BEGIN
       Inc(ToFrom[city]);
       FOR i:=1 TO CITYCNT DO
        BEGIN
         IF (city<>i) AND (FromCnt[i]>0) AND NOT done[i] THEN
          BEGIN
           Dec(ToFrom[i]);
           Solve(i,city,level+1,false);
           Inc(ToFrom[i]);
          END;
        END;
       Dec(ToFrom[city]);
      END;
    END
   ELSE
    IF city=ENDCITY THEN
     BEGIN
      IF BestXWay>XWay THEN            (* This is the best yet found *)
       BEGIN
        BestAnswer:=Copy(Answer,2,level*2+2-1);
        BestXWay:=XWay;
       END;
     END
    ELSE
     Solve(ENDCITY,city,level+1,false);
 XWay:=saveXWay;
END;

BEGIN (* program *)
 ReadOrders('INPUT.TXT',OrderCnt,Orders);
 CalcTables(Orders,FromCnt,ToFrom,MinWay,OrderCnt);

 XWay:=0;
 BestXWay:=29999;
 FillChar(Answer,SizeOf(Answer),' ');
 Answer[0]:=#255;                   (* Set length of string to 255 *)
 OrdersLeft:= OrderCnt;

 Solve(ENDCITY,ENDCITY,0,true);
   (* ENDCITY -> ENDCITY: imaginary null-flight
      true: imagine we had cargo justifies that flight
      0: level of recursion *)

 Assign(f, 'OUTPUT.TXT');           (* Output the best solution *)
 Rewrite(f);
 WriteLn(f, MinWay+BestXWay);
 WriteLn(f, BestAnswer);
 Close(f);
END.


