/*
**		Sample solution to problem 5 from IOI6 day2
*/
#include <fstream.h>

const int MAX_SIDE = 30;
const int MAX_FRAMES = 26;
const int FALSE = 0;
const int TRUE = !FALSE;

int h, w;

char sheet[MAX_SIDE][MAX_SIDE];

int num_frames = 0;

void get_input()
{
	char tmp;
	ifstream in("input.txt");

	in >> h >> w;

	for(int y = 0; y < h; y++)
		for(int x = 0; x < w; x++)
			{
			in >> tmp;

			if (tmp == '.')
				sheet[y][x] = MAX_FRAMES;
			else
				{
				tmp -= 'A';
				if (tmp + 1 > num_frames) num_frames = tmp + 1;
				sheet[y][x] = tmp;
				}
			}
}

int left[MAX_FRAMES], right[MAX_FRAMES], top[MAX_FRAMES], bottom[MAX_FRAMES];

// Find the left, right top, and bottom sides of the frame.
void find_sides()
{
	// Initialise each frame
	for (int n = 0 ; n < 26; n++)
		{
		left[n] = bottom[n] = MAX_SIDE;
		right[n] = top[n] = 0;
		}

	// Work your way through the sheet, updating values for left, right etc
	for(int y = 0 ; y < h; y++)
		for(int x = 0 ; x < w; x++)
			if (sheet[y][x] != MAX_FRAMES)
				{
				// Update sides of this frame
				if (left[sheet[y][x]] > x) left[sheet[y][x]] = x;
				if (right[sheet[y][x]] < x) right[sheet[y][x]] = x;
				if (bottom[sheet[y][x]] > y) bottom[sheet[y][x]] = y;
				if (top[sheet[y][x]] < y) top[sheet[y][x]] = y;
				}
}

int gone[MAX_FRAMES];

/*
**		Is this frame on top
*/
int ontop(int frame)
{
	int count = 0;

	// Put left, right, bottom and top into local variables
	int l = left[frame];
	int r = right[frame];
	int b = bottom[frame];
	int t = top[frame];

	// Look on the top and bottom rows for a frame placed on this
	for(int x = l; x <= r; x++)
		{
		// Bottom row
		if (sheet[b][x] != frame)
			// Square not part of the frame, have we counted it ?
			if(!gone[sheet[b][x]])
				count++;

		// Top row
		if (sheet[t][x] != frame)
			// Square not part of the frame, have we counted it ?
			if(!gone[sheet[t][x]])
				count++;
		}

	// Look on the left and right columns
	for(int y = b; y <= t; y++)
		{
		// Left side
		if (sheet[y][l] != frame)
			if(!gone[sheet[y][l]])
				count++;

		// Right side
		if (sheet[y][r] != frame)
			if(!gone[sheet[y][r]])
				count++;
		}

	return count == 0;	// This frame is on top only if nought else on top
}

void main()
{
	get_input();

	find_sides();

	int ordered[MAX_FRAMES];

	for(int frame = 0; frame < num_frames; frame++)
		ordered[frame] = MAX_FRAMES;

	// Find the next uncovered frame, and stick into the ordered array.
	// When done, mark that frame as gone, so that it won't be counted again.
	for(int num_done = 0; num_done < num_frames; num_done++)
		for(frame = 0; frame < num_frames; frame++)
			if (ontop(frame) && !gone[frame])
				{
				ordered[num_done] = frame;
				gone[frame] = TRUE;		// Mark as out of action
				break;
				}

	for(frame = 0; frame < num_frames; frame++)
		cout << (char) (ordered[frame] + 'A');
	cout << endl;
}

