/*
	buses.c

	Gunnar Andersson, The Scientific Committee
	1994-07-08

	IOI 1994 Day 2, Problem 2

	The idea used in the solution is to generate a list of all bus lines
	possible given the set of input data. The list is sorted so that
	the bus routes which pass the highest number of times are placed at the
	top of the list. The search is done recursively so that if the the bus
	with index i in the list is chosen at depth n, then the bus chosen at
	depth n+1 has index >= i. The sorting makes it possible to place good
	upper	and lower bounds on how many times the bus chosen at the current
	depth must pass.
*/

#include <dos.h>
#include <stdio.h>
#include <stdlib.h>

#define MAXTIMES 300
#define MAXLEVELS 50
#define START 0
#define STOP 59

typedef struct {
	int start;
	int period;
	int count;
} BusRoute;

int given_times,routes;
int times[MAXTIMES];
int diff[MAXLEVELS],start[MAXLEVELS],index[MAXLEVELS],count[MAXLEVELS];
int remaining[60];
BusRoute *busroutes;

/* Read the times from the file INPUT.TXT */

void get_data() {
	FILE *stream;
	int i;

	stream = fopen("input.txt","r");
	fscanf(stream,"%d",&given_times);
	for (i = 0; i < given_times; i++)
		fscanf(stream,"%d",&times[i]);
	fclose(stream);
}

/* Write the solution found to the file OUTPUT.TXT. */

void write_solution(int buses) {
	FILE *stream;
	int i;

	stream = fopen("output.txt","w");
	for (i = 1; i <= buses; i++)
		fprintf(stream,"%2d     %2d\n",start[i],diff[i]);
	fclose(stream);
	exit(0);
}

/* Search recursively for a solution which is valid and contains a specified
	number of buses. This number is given by the parameter buses. */

void search(int level,int buses,int stops) {
	int pos,i,j,mincount,maxcount;
	FILE *stream;

	mincount = stops/(buses-level+1);
	if (mincount*(buses-level+1) != stops)
		mincount++;
	maxcount = stops-(buses-level)*2;
	for (i = index[level-1]; i < routes; i++)
		if (busroutes[i].count >= mincount && busroutes[i].count <= maxcount) {

/* Check if the bus route given by the index i is valid */

			pos = busroutes[i].start;
			while ((pos <= STOP) && (remaining[pos] > 0))
				pos+=busroutes[i].period;
			if (pos > STOP) {

/* Decrease the counters for how many stops that remain to be covered
	for each minute. */

				pos = busroutes[i].start;
				while (pos <= STOP) {
					remaining[pos]--;
					pos+=busroutes[i].period;
					stops--;
				}
				start[level] = busroutes[i].start;
				diff[level] = busroutes[i].period;
				count[level] = busroutes[i].count;
				index[level] = i;
				if (level != buses)
					search(level+1,buses,stops);
				else
					if (!stops)
						write_solution(buses);

/* Decrease the counters for how many stops that remain to be covered
	for each minute. */

				pos = busroutes[i].start;
				while (pos <= STOP) {
					remaining[pos]++;
					pos+=busroutes[i].period;
					stops++;
				}
			}
		}
}

/* Generate all bus routes possible given the data read from the file
	INPUT.TXT. The parameter write indicates whether the bus routes should
	be stored into memory or not. */

void get_busroutes(int write) {
	int i,j,k;
	int count;
	int done;
	BusRoute temp;

	routes = 0;
	for (i = 1; i <= STOP-START; i++)
		for (j = 0; j < i; j++) {
			k = j;
			while (k <= STOP && remaining[k] > 0)
				k+=i;
			count = 1+(STOP-j)/i;
			if (count >= 2 && k > STOP) {
				if (write) {
					busroutes[routes].period = i;
					busroutes[routes].start = j;
					busroutes[routes].count = count;
				}
				routes++;
			}
		}
	done = 0;
	while (!done) {
		done = 1;
		for (i = 0; i < STOP-1; i++)
			if (busroutes[i].count < busroutes[i+1].count) {
				temp = busroutes[i];
				busroutes[i] = busroutes[i+1];
				busroutes[i+1] = temp;
				done = 0;
			}
	}
}

void main() {
	int i;

	get_data();
	for (i = START; i <= STOP; i++)
		remaining[i] = 0;
	for (i = 0; i < given_times; i++)
		remaining[times[i]]++;
	get_busroutes(0);
	busroutes = (BusRoute *) malloc(routes*sizeof(BusRoute));
	get_busroutes(1);
	index[0] = 0;
	for (i = 1; i < (given_times+1)/2; i++)
		search(1,i,given_times);
}
