/*
 * Number triangle problem of The 6th International Olympiad in Informatics
 *
 * Written by: Fredrik Hubinette (hubbe@lysator.liu.se)
 *
 */

/* The trick in this problem is to add the numbers from the bottom, and
 * remembering which way gave the larger sum. This reduces the time-
 * complexity of this program from O( 2^height ) to O( height^2 )
 * which is a massive gain.
 */

#include <stdio.h>
#include <stdlib.h>


int main()
{
  int height,storage;
  int x,y;
  FILE *f;
  int *sums;

  /* read input */
  f=fopen("input.txt","r");
  if(!f)
  {
    perror("No such file");
    exit(1);
  }

  if(!fscanf(f,"%d",&height))
  {
    fprintf(stderr,"Couldn't read file.\n");
    exit(1);
  }

  if(height<1)
  {
    fprintf(stderr,"Error in in input, height < 1\n");
    exit(1);
  }
  storage=height*height;
  sums=(int *)malloc(storage*sizeof(int));    /* the sums on the way */

  if(!sums)
  {
    fprintf(stderr,"Couldn't allocate memory.\n");
    exit(1);
  }

  /* read input */
  for(y=0;y<height;y++)
  {
    for(x=0;x<=y;x++)
    {
      if(!fscanf(f,"%d",&(sums[y*height+x])))
      {
	perror("Error in indata file");
	exit(1);
      }
    }
  }

  fclose(f);

  /* loop from the second last line, finding which of the
   * branches underneath that has the larger sum, store that,
   * and calculating the sum this position, when we reach
   * the top, we will know the largest sum.
   */
  for(y=height-2;y>=0;y--)
  {
    for(x=0;x<=y;x++)
    {
      if( sums[(y+1)*height+x] > sums[(y+1)*height+x+1] )
	sums[y*height+x]+=sums[(y+1)*height+x];
      else
	sums[y*height+x]+=sums[(y+1)*height+x+1];
    }
  }

  f=fopen("output.txt","w");
  if(!f)
  {
    fprintf(stderr,"Couldn't open output file.\n");
    exit(1);
  }
  fprintf(f,"%d\n",sums[0]);

  fclose(f);
  free((char *)sums);
  return 0;
}
