{
   Solution for problem 5 day 1.

   By Fredrik Huss of Sweden in 1994.

   ========================================================================
}

{$M 60000,0,0} { Just to be sure... }

Const
   maxx = 8;   { Maximum number of columns }
   maxy = 4;   { Maximum number of rows }
   maxn = 16;  { Maximum of maxx*maxy }
   maxd = 256; { Maximum number of moves }

   FoldTypeLR : Array[False..True] of Char = ('L','R');
   FoldTypeUD : Array[False..True] of Char = ('U','L');

Type
   MapType = Array[1..maxx,1..maxy] of String[maxn];

Var
   xsize, ysize : Integer;
   sol,sol_backwards : String;
   m : MapType;
   i,j,k : Integer;
   c : Char;
   f : Text;
   solution : Array[1..maxd] of String[3];

Procedure TryMove(VAR m : MapType; xs,ys,n : Integer); Forward;

Procedure Fold(VAR m : MapType; xs,ys,x,y : Integer; xside,yside : Boolean; n : Integer);
{
   Input: ?side : False = Fold (left part to the right)/(upper part downwards)
                  True  = Fold (right part to the left)/(lower part upwards)
          m     : The map
          xs,ys : Mapsize
          x,y   : Tells where to fold. One of them is always zero
          n     : Recursion depth
}
Var xs2,ys2 : Integer;
    i,j : Integer;
    x1,y1,x2,y2 : Integer;
    s1,s2,s3 : String[maxn];
    tm : MapType;
    t1,t2 : Boolean;
Begin
   {
   Store description of move.
   }
   If x>0 then
      solution[n]:='V'+Char(x+48)+FoldTypeLR[xside]
   else
      solution[n]:='H'+Char(y+48)+FoldTypeUD[yside];

   {
   Calculate new map size.
   }
   If x>0 then Begin
      xs2:=xs-x;
      If x>xs2 then xs2:=x;
   End else xs2:=xs;
   If y>0 then Begin
      ys2:=ys-y;
      If y>ys2 then ys2:=y;
   End else ys2:=ys;

   {
   Clear new map.
   }
   FillChar(tm,Sizeof(tm),0);

   {
   Fold the old map m into the new map tm.
   }
   For i:=1 to ys2 do { For each square in the new map... }
      For j:=1 to xs2 do Begin
         {
         Find the two squares in the old map that will compose the new square.
         }
         If x=0 then Begin
            x1:=j; x2:=j; { No fold in the x direction }
         End else If xside then Begin { Fold to the left or to the right? }
            x1:=x-(xs2-j);
            x2:=x+1+(xs2-j);
         End else Begin
            x1:=x+j;
            x2:=x+1-j;
         End;
         If y=0 then Begin
            y1:=i; y2:=i; { No fold in the y direction }
         End else If yside then Begin { Fold upwards or downwards? }
            y1:=y-(ys2-i);
            y2:=y+1+(ys2-i);
         End else Begin
            y1:=y+i;
            y2:=y+1-i;
         End;

         {
         Check if the squares are inside the old map.
         }
         t1:=(x1>=1) and (x1<=xs) and (y1>=1) and (y1<=ys);
         t2:=(x2>=1) and (x2<=xs) and (y2>=1) and (y2<=ys);

         {
         Get the contents of the old squares and reverse the second one.
         }
         If t1 then s1:=m[x1,y1] else s1:='';
         If t2 then Begin
            s3:=m[x2,y2];
            For k:=1 to Length(s3) do
              s2[k]:=s3[Length(s3)-k+1];
            s2[0]:=s3[0];
         End else s2:='';

         {
         Optimization: You must be able to find the square in the solution.
         }
         If Pos(s2+s1,sol)+Pos(s2+s1,sol_backwards)=0 then Exit;

         {
         Store the square in the new map.
         }
         tm[j,i]:=s2+s1;
      End;
   TryMove(tm,xs2,ys2,n+1);
End;

Procedure TryMove(VAR m : MapType; xs,ys,n : Integer);
{
   Input: m     : The map
          xs,ys : Mapsize
          n     : Recursion depth
}
Var i : Integer;
Begin
   If (xs=1) and (ys=1) then Begin
      {
      We're almost done. Check the solution.
      }
      If m[1,1]=sol then Begin
         For i:=1 to n-1 do
            Write(f,solution[i],' ');
         Writeln(f);
      End;
   End else Begin
      {
      Try all folds in the x direction.
      }
      For i:=1 to xs-1 do Begin
         Fold(m,xs,ys,i,0,True,False,n);
         Fold(m,xs,ys,i,0,False,False,n);
      End;

      {
      Try all folds in the y direction.
      }
      For i:=1 to ys-1 do Begin
         Fold(m,xs,ys,0,i,False,True,n);
         Fold(m,xs,ys,0,i,False,False,n);
      End;
   End;
End;

Begin
   Assign(f,'INPUT.TXT'); Reset(f);
   Readln(f,ysize);
   Readln(f,xsize);
   Readln(f,sol);
   Close(f);

   {
   Turn the solution backwards into solback.
   }
   sol_backwards[0]:=sol[0];
   For i:=1 to Length(sol) do
      sol_backwards[i]:=sol[Length(sol)-i+1];

   {
   Fill the original map.
   }
   c:='A';
   For i:=1 to ysize do
      For j:=1 to xsize do Begin
         m[j,i]:=c;
         Inc(c);
      End;

   {
   Try all possible ways to fold the map...
   }
   Assign(f,'OUTPUT.TXT'); Rewrite(f);
   TryMove(m,xsize,ysize,1);
   Close(f);
End.
