#include <stdio.h>
#include <math.h>
#include <stdlib.h>

struct Corner
{
  int iX, iY;   /* coords */
  int iNext;    /* next point to get shortest path */
  int iHouse;	/* which house this corner belongs to */
  double adLen; /* length to 100,100 */
};

#define EPSILON 1e-7

#define FALSE 0
#define TRUE  1

#define SQR(x)  ((x)*(x))
#define LEN(x1,y1,x2,y2) sqrt( SQR(x2-x1)+SQR(y2-y1))


int iNumHouse, iNumCorn;
FILE *fhIn, *fhOut;
int rgi[100][2]; /* lower left corner of houses */
struct Corner rgCorn[200];

void vReadIn()
{
  int i;

  fscanf (fhIn, "%d", &iNumHouse);
  for (i=0; i<iNumHouse; i++)
    fscanf (fhIn, "%d %d", &rgi[i][0], &rgi[i][1]);
}

int iCmp(const int rgi1[2], const int rgi2[])
{
  if (rgi1[0]+rgi1[1] < rgi2[0]+rgi2[1])
    return -1;
  else if (rgi1[0]+rgi1[1] > rgi2[0]+rgi2[1])
    return +1;
  return 0;
}

void vSort()
{
  qsort (rgi, iNumHouse, 2*sizeof(int), (int (*)(const void *, const void *)) iCmp);
}

void vInitCorner()
{
  int i;

  iNumCorn = 0;

  rgCorn [iNumCorn].iX = 0;
  rgCorn [iNumCorn].iY = 0;
  rgCorn [iNumCorn].iHouse = -1;
  rgCorn [iNumCorn].iNext = -1;
  iNumCorn++;

  for (i=0; i<iNumHouse; i++)
  {
    rgCorn [iNumCorn].iX = rgi[i][0]+5;
    rgCorn [iNumCorn].iY = rgi[i][1];
    rgCorn [iNumCorn].iHouse = i;
    rgCorn [iNumCorn].iNext = -1;
    iNumCorn++;
    rgCorn [iNumCorn].iX = rgi[i][0];
    rgCorn [iNumCorn].iY = rgi[i][1]+5;
    rgCorn [iNumCorn].iHouse = i;
    rgCorn [iNumCorn].iNext = -1;
    iNumCorn++;
  }

  rgCorn [iNumCorn].iX = 100;
  rgCorn [iNumCorn].iY = 100;
  rgCorn [iNumCorn].iHouse = -1;
  rgCorn [iNumCorn].iNext = iNumCorn;
  rgCorn [iNumCorn].adLen = 0.0;
  iNumCorn++;
}



static int fOkay2Go (int iCorn, int iX, int iY)
{
  int i, iDX, iDY;
  long l1, l2;

  iDX = iX-rgCorn[iCorn].iX;
  iDY = iY-rgCorn[iCorn].iY;

  if (iDX<0 || iDY<0)
    return FALSE;

  for (i=rgCorn[iCorn].iHouse+1; i<iNumHouse; i++)
  {
    if (iX+iY > rgi[i][0] + rgi[i][1] + 5)
    {
      l1 = iDX * (rgi[i][1]+5-rgCorn[iCorn].iY) -
	   iDY * (rgi[i][0]  -rgCorn[iCorn].iX);
      l2 = iDX * (rgi[i][1]  -rgCorn[iCorn].iY) -
	   iDY * (rgi[i][0]+5-rgCorn[iCorn].iX);
      if ( (l1<0 && l2>0 ) || (l1>0 && l2<0) )
      {
// printf ("FALSE:: (%d %d)-(%d %d) thru house (%d %d)\n",
//	   rgCorn[iCorn].iX, rgCorn[iCorn].iY, iX, iY,
//	   rgi[i][0], rgi[i][1] );
	return FALSE;
      }
    }
  }

  return TRUE;	/* passed all tests */
}


static double adShortPath(int iCorn)
{
  int i;
  double ad;
  long lDX1, lDY1, lDX2, lDY2;

  if (rgCorn[iCorn].iNext>=0)
    return rgCorn[iCorn].adLen;

  rgCorn[iCorn].adLen = 1e100;

  for (i=iNumCorn-1; i>iCorn; i--)
  {
    if (fOkay2Go (iCorn, rgCorn[i].iX, rgCorn[i].iY ))
    {
      ad = LEN (rgCorn[iCorn].iX, rgCorn[iCorn].iY,
		rgCorn[i    ].iX, rgCorn[i    ].iY )
	   + adShortPath (i);

      if (fabs(ad-rgCorn[iCorn].adLen) < EPSILON && rgCorn[iCorn].iNext>=0)
      {
	lDX1 = rgCorn[rgCorn[iCorn].iNext].iX - rgCorn[iCorn].iX;
	lDY1 = rgCorn[rgCorn[iCorn].iNext].iY - rgCorn[iCorn].iY;
	lDX2 = rgCorn[i].iX - rgCorn[iCorn].iX;
	lDY2 = rgCorn[i].iY - rgCorn[iCorn].iY;
//	printf ("a- Length is same! %ld %ld %ld %ld\n", lDX1, lDY1, lDX2, lDY2);
/*	if (lDY1 && lDY2 && lDX1 && lDX2)
	  if (lDY1*lDX2 != lDY2*lDX1)
	    printf ("WARNING!!!! - several solutions!!!!!!!!\n");
*/
      }

      if (ad < rgCorn[iCorn].adLen)
      {
	rgCorn[iCorn].adLen = ad;
	rgCorn[iCorn].iNext = i;
	if (i == iNumCorn-1)
	  return rgCorn[iCorn].adLen;
      }
    }
  }

/****
  if (rgCorn[iCorn].adLen > 1e99)
  {
    printf ("iCorn=%d - (%d %d) couldn't go home...\n", iCorn,
      rgCorn[iCorn].iX, rgCorn[iCorn].iY);
  }
****/

  return rgCorn[iCorn].adLen;
}


static void vCalcRoute()
{
  int i,iCorn;
  double ad;

  ad = adShortPath(0);
  printf ("Shortest path is : %.6f\n", ad);

  for (i=iCorn=0; i<100; i++)
  {
    if (iCorn<0 || iCorn>=iNumCorn)
    {
      printf ("invalid iCorn=%d\n", iCorn);
      break;
    }

    printf ("%d : %d %d\n", iCorn, rgCorn[iCorn].iX, rgCorn[iCorn].iY);
    if (iCorn==iNumCorn-1)
      break;
    iCorn = rgCorn[iCorn].iNext;
  }
}

void vOutput()
{
  int i,iCorn;

  for (i=iCorn=0; i<100; i++)
  {
    if (iCorn<0 || iCorn>=iNumCorn)
      break;

    fprintf (fhOut, "%d %d\n", rgCorn[iCorn].iX, rgCorn[iCorn].iY);
    if (iCorn==iNumCorn-1)
      break;
    iCorn = rgCorn[iCorn].iNext;
  }
}

void vStart()
{
  vReadIn();
  vSort();
  vInitCorner();
  vCalcRoute();
  vOutput();
}


void main()
{
  if ( (fhIn=fopen("input.txt","rt")) != NULL)
  {
    if ( (fhOut=fopen("output.txt","wt")) != NULL)
    {
      vStart();
      fclose (fhOut);
    }
    else
      printf ("Couldn't open output.txt\n");
    fclose (fhIn);
  }
  else
    printf ("Couldn't open input.txt\n");
}
