/**************************************************************
*                                                             *
*          6th International Olympiad in Informatics          *
*                                                             *
*                         The clocks                          *
*                                                             *
*                 Solution by Marcus Comstedt                 *
*                                                             *
**************************************************************/

#include <stdio.h>

/*
   Premise for the algorithm:
   There is no sequence of moves where no specific move is used more
   than three times that will restore the board to its initial position
   except for the trivial sequence with no moves.

   Consequence of the premise:
   There can be no more than one sequence of moves (order of moves is
   not considered as it is irrelevant) that will solve the given problem
   without using a specific move more than three times.  It is therefore
   sufficient to find one such sequence to find the shortest.  Should a
   sequence be found where one specific move is used four or more times,
   a simple modulo 4 operation will transform it into the desired solution.
*/

#define LO 0x15555  /* Mask to isolate the low bits, used in the parallell
		       addition below                                      */

/* Sequences of moves that will turn exactly one clock one step
   counterclockwise.  Each long is such a sequence, and there is
   one for each clock.  The sequences are represented as groups of two
   bits specifying the number of times a specific move should be used  */

long solve[] = {
  0x09955,0x33666,0x18595,0x36276,0x26766,0x27627,0x15949,0x26673,0x15598
};

void main()
{
  int i,j;
  long turns=0;
  FILE *f;


  if(!(f=fopen("input.txt", "rt"))) {
	 fprintf(stderr, "Cannot open INPUT.TXT!\n");
	 exit(10);
  }

  /* Find an individual solution for each clock, and add these together
	  using modulo 4 parallell addition                                  */

  for(i=0;i<9;i++) {
	 fscanf(f, "%d", &j);
	 if(j&1) turns^=solve[i]^(solve[i]&turns&LO)<<1;  /* add sequence once */
	 if(j&2) turns^=(solve[i]&LO)<<1;                 /* add sequence twice */
  }

  fclose(f);

  if(!(f=fopen("output.txt", "wt"))) {
	 fprintf(stderr, "Cannot create OUTPUT.TXT!\n");
	 exit(10);
  }

  /* Print the resulting sequence of moves which is the solution to the
	  given problem                                                      */

  for(i=1;i<10;i++) {
	 for(j=turns&3;j--;)
		fprintf(f, "%d ",i);
	 turns>>=2;
  }
  fprintf(f, "\n");

  fclose(f);
  exit(0);
}
