/*********************************************
   6 th International Olympiad in Informatics

   Problem 3, day 1: Prime squares

   Martin Fahlgren
**********************************************/

#include <stdio.h>
#include <stdlib.h>

#define LIMIT 99999

unsigned long *primtab;
unsigned numPrimes;

unsigned SumOfDigits(unsigned long num)
{
  unsigned sum = 0;

  while (num) {
    sum = sum + num % 10;
    num /= 10;
  }
  return sum;
}


unsigned ReducedPrimeTable(unsigned digitSum)
/* 5 digit primes with given digit sum */
{
  char *sv;
  unsigned long pix, i, k, prime, maxp;
  unsigned num;

  maxp = (LIMIT + 1)/2;
  sv = (char *) malloc(maxp);
  if (!sv) {
    fprintf(stderr, "Out of memory (1)\n");
    exit(0);
  }
  /* Sieve of Eratosthenes */
  pix = 0;
  for (i=0; i < maxp; i++)
    sv[i]=1;
  for (i=0; i < maxp; i++)
    if (sv[i]) {
      prime = i + i + 3;
      for (k=i + prime; k < maxp; k+=prime)
	sv[k] = 0;
      if (prime > 10000)
	pix++;
    }
  primtab = (unsigned long *) malloc(pix/4*sizeof(unsigned long));
  if (!primtab) {
    fprintf(stderr, "Out of memory (2)\n");
    exit(0);
  }
  for (i=0; i+i+3 <= 10000; i++)
    ;
  num=0;
  for (; i < maxp; i++) {
    prime = i + i + 3;
   if (sv[i] && SumOfDigits(prime) == digitSum)
     primtab[num++] = prime;
  }
  primtab[num] = 1000000;   /* sentinel */
  free(sv);
  return num;
}


unsigned GetMinIndex(unsigned long minval)
{
  unsigned low, high, mid;

  low = 0;
  high = numPrimes - 1;
  while (low < high) {
    mid = (low + high)/2;
    if (minval > primtab[mid])
      low = mid + 1;
    else
      high = mid;
  }
  if (minval <= primtab[low])
    return low;
  else
    return low+1;
}


unsigned GetIndex0(unsigned i0, unsigned long maxval)
{  /* first row and column can not contain zeros */
  int ok = 0;
  unsigned long temp;

  while ((temp = primtab[i0]) < maxval && !ok) {
    ok = 1;
    while (ok && temp) {
      ok = temp % 10;  /* Not 0 */
      temp /= 10;
    }
    if (!ok) i0++;
  }
  return i0;
}

unsigned DiagonalsOK(unsigned long *r, unsigned digitSum)
{
  unsigned long num;
  unsigned i;

  /* 0,0 -> 4,4 */
  num = r[0]/10000 + (r[1]/1000) % 10 + (r[2]/100) % 10 + (r[3]/10) % 10 + r[4] % 10;
  if (num != digitSum)
    return 0;
  num = r[4]/10000 + (r[3]/1000) % 10 + (r[2]/100) % 10 + (r[1]/10) % 10 + r[0] % 10;
  if (num != digitSum)
    return 0;
  /* 1st diagonal */
  num = 10000*(r[0]/10000) + 1000*((r[1]/1000) % 10) +
	100*((r[2]/100) % 10) + 10*((r[3]/10) % 10) + r[4] % 10;
  i=GetMinIndex(num);
  if (num != primtab[i])
    return 0;
  num = 10000*(r[4]/10000) + 1000*((r[3]/1000) % 10) +
	100*((r[2]/100) % 10) + 10*((r[1]/10) % 10) + r[0] % 10;
  i=GetMinIndex(num);
  return (num == primtab[i]);
}


void main()
{
  unsigned long row[5], col[5], minrow[5], maxrow[5], mincol[5], maxcol[5];
  unsigned long firstDigit, mincol1, minrow2, mincol2, minrow3, mincol3, minrow4;
  unsigned i, i0, j0, i1, j1, i2, j2, i3, j3, i4, j4, digitSum;
  unsigned minix;
  FILE *inFile, *outFile;

  inFile = fopen("input.txt", "rt");
  outFile = fopen("output.txt", "wt");
  fscanf(inFile, "%u", &digitSum);
  numPrimes = ReducedPrimeTable(digitSum);
  fscanf(inFile, "%lu", &firstDigit);
  minrow[0] = 10000*firstDigit;	/* tempory */
  maxrow[0] = minrow[0] + 10000;
  maxcol[0] = maxrow[0];
  /* In row[0] and col[0] the 4 last digits must be != 0 */
  minrow[0] += 1111;
  minix = GetIndex0(GetMinIndex(minrow[0]), maxrow[0]);
  /* minrow[0] = primtab[minix];  mincol[0] = minrow[0]; */
  /* Select row[0] */
  for(i0=minix; primtab[i0] < maxrow[0]; i0=GetIndex0(++i0, maxrow[0])) {
    row[0]=primtab[i0];
    mincol1=((row[0]/1000) % 10)*10000;
    mincol2=((row[0]/100) % 10)*10000;
    mincol3=((row[0]/10) % 10)*10000;
    for(j0=minix; primtab[j0] < maxcol[0]; j0=GetIndex0(++j0, maxcol[0])) {
      col[0]=primtab[j0];
      minrow2=((col[0]/100) % 10)*10000;
      minrow3=((col[0]/10) % 10)*10000;
      minrow4=(col[0] % 10)*10000;
      /* select row[1] */
      minrow[1]=((col[0]/1000) % 10)*10000;
      maxrow[1]=minrow[1] + 10000;
      for(i1=GetMinIndex(minrow[1]); primtab[i1] < maxrow[1]; i1++) {
	row[1]=primtab[i1];
	/* col[1]*/
	mincol[1]=mincol1 + ((row[1]/1000) % 10)*1000;
	maxcol[1]=mincol[1] + 1000;
	for(j1=GetMinIndex(mincol[1]); primtab[j1] < maxcol[1]; j1++) {
	  col[1]=primtab[j1];
	  /* row[2] */
	  minrow[2]=minrow2 + ((col[1]/100) %10)*1000;
	  maxrow[2]=minrow[2] + 1000;
	  for(i2=GetMinIndex(minrow[2]); primtab[i2] < maxrow[2]; i2++) {
	    row[2]=primtab[i2];
	    /* col[2]*/
	    mincol[2]=mincol2 + ((row[1]/100) % 10)*1000 + ((row[2]/100) % 10)*100;
	    maxcol[2]=mincol[2] + 100;
	    for (j2=GetMinIndex(mincol[2]); primtab[j2] < maxcol[2]; j2++) {
	      col[2]=primtab[j2];
	      /* row[3] */
	      minrow[3]=minrow3 + ((col[1]/10) %10)*1000 + ((col[2]/10) %10)*100;
	      maxrow[3]=minrow[3] + 100;
	      for (i3=GetMinIndex(minrow[3]); primtab[i3] < maxrow[3]; i3++) {
		row[3]=primtab[i3];
		/* col[3]*/
		mincol[3]=mincol3 + ((row[1]/10) % 10)*1000 + ((row[2]/10) % 10)*100 +
			  ((row[3]/10) % 10)*10;
		maxcol[3]=mincol[3] + 10;
		for (j3=GetMinIndex(mincol[3]); primtab[j3] < maxcol[3]; j3++) {
		  col[3]=primtab[j3];
		  /* row[4] */
		  minrow[4]=minrow4 + (col[1]%10)*1000 + (col[2]%10)*100 +
			   (col[3]%10)*10;
		  maxrow[4]=minrow[4] + 10;
/* For a given digit sum there are at most 1 prime between minrow[4] && maxrow[4]
   because every other unit digit gives a different digit sum */
		  i4=GetMinIndex(minrow[4]);
		  if (primtab[i4] < maxrow[4]) {
		    row[4]=primtab[i4];
		    /* col[4]*/
		    col[4]=(row[0]%10)*10000 + (row[1]%10)*1000 +
			   (row[2]%10)*100 + (row[3]%10)*10 + row[4] % 10;
		    j4=GetMinIndex(col[4]);
		    if (col[4] == primtab[j4] && DiagonalsOK(row, digitSum)) {
		      for (i=0; i < 5; i++)
			fprintf(outFile, "%lu\n", row[i]);
		      fprintf(outFile, "\n");
		    }
		  }
		}
	      }
	    }
	  }
	}
      }
    }
  }
  fclose(outFile);
}
